@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 import re
 import urllib2
+import logging
 from types import UnicodeType
 
 from django.db import models
@@ -8,24 +9,35 @@ from django.utils.translation import ugettext_lazy as _
 
 from webcall import MultipartPostHandler
 
+logger = logging.getLogger(__name__)
+
 # Mailman-Messages for a successfull subscription
 SUBSCRIBE_MSG = (
     u'Erfolgreich eingetragen', # de
     u'Successfully subscribed', # en
+    u'Abonnement r\xe9ussi', # fr
 )
 
 # Mailman-Messages for successfully remove from a list
 UNSUBSCRIBE_MSG = (
     u'Erfolgreich beendete Abonnements', # de
     u'Successfully Removed', # en
+    u'Successfully Unsubscribed', # also en
+    u'R\xe9siliation r\xe9ussie', # fr
 )
 
 # Mailman-Messages for a failed remove from a list
 NON_MEMBER_MSG = (
     u'Nichtmitglieder können nicht aus der Mailingliste ausgetragen werden', # de
     u'Cannot unsubscribe non-members', # en
+    u"Ne peut r\xe9silier l'abonnement de non-abonn\xe9s ", # fr
 )
 
+# To control user form unsubscription
+UNSUBSCRIBE_BUTTON = {
+    'fr' : 'Résilier',
+}
+
 # Definition from the Mailman-Source ../Mailman/Default.py
 LANGUAGES = (
     ('utf-8',       _('Arabic')),
@@ -126,14 +138,16 @@ class List(models.Model):
         else:
             raise Exception('Could not find member-information')
 
-        return (unicode(msg), unicode(member))
+        msg = msg.decode(self.encoding)
+        member = member.decode(self.encoding)
+        return (msg, member)
 
     def __parse_member_content(self, content, encoding='iso-8859-1'):
         if not content:
             raise Exception('No valid Content!')
-
         members = []
-        letters = re.findall('\?letter=[a-z]', content)
+        letters = re.findall('letter=\w{1}', content)
+        chunks = re.findall('chunk=\d+', content)
         input = re.findall('name=".+_realname" type="TEXT" value=".*" size="[0-9]+" >', content)
         for member in input:
             info = member.split('" ')
@@ -142,7 +156,8 @@ class List(models.Model):
             email = unicode(email, encoding)
             realname = unicode(realname, encoding)
             members.append([realname, email])
-        return (letters, members)
+        letters = set(letters)
+        return (letters, members, chunks)
 
     def get_admin_moderation_url(self):
         return '%s/admindb/%s/?adminpw=%s' % (self.main_url, self.name,
@@ -188,12 +203,87 @@ class List(models.Model):
         opener = urllib2.build_opener(MultipartPostHandler(self.encoding))
 
         all_members = []
-        content = opener.open(url, data).read()
-        (letters, members) = self.__parse_member_content(content, self.encoding)
-        all_members.extend(members)
-        for letter in letters[1:]:
+        try:
             content = opener.open(url, data).read()
-            (letters, members) = self.__parse_member_content(content, self.encoding)
+        except urllib2.HTTPError as error:
+            logger.error("%s %s" % (error, url))
+            return []
+    
+        (letters, members, chunks) = self.__parse_member_content(content, self.encoding)
+        all_members.extend(members)
+        for letter in letters:
+            url_letter = u"%s?%s" %(url, letter)
+            content = opener.open(url_letter, data).read()
+            (letters, members, chunks) = self.__parse_member_content(content, self.encoding)
             all_members.extend(members)
+            for chunk in chunks[1:]:
+                url_letter_chunk = "%s?%s&%s" %(url, letter, chunk)
+                content = opener.open(url_letter_chunk, data).read()
+                (letters, members, chunks) = self.__parse_member_content(content, self.encoding)
+                all_members.extend(members)
+
+        members = {}
+        for m in all_members:
+            email = m[1].replace(u"%40", u"@")
+            members[email] = m[0]
+        all_members = [(email, name) for email, name in members.items()]
         all_members.sort()
         return all_members
+
+    def user_subscribe(self, email, password, language='fr', first_name=u'', last_name=u''):
+
+        url = '%s/subscribe/%s' % (self.main_url, self.name)
+
+        password = check_encoding(password, self.encoding)
+        email = check_encoding(email, self.encoding)
+        first_name = check_encoding(first_name, self.encoding)
+        last_name = check_encoding(last_name, self.encoding)
+        name = '%s %s' % (first_name, last_name)
+
+        SUBSCRIBE_DATA['email'] = email
+        SUBSCRIBE_DATA['pw'] = password
+        SUBSCRIBE_DATA['pw-conf'] = password
+        SUBSCRIBE_DATA['fullname'] = name
+        SUBSCRIBE_DATA['language'] = language
+        opener = urllib2.build_opener(MultipartPostHandler(self.encoding, True))
+        request = opener.open(url, SUBSCRIBE_DATA)
+        content = request.read()
+        for status in SUBSCRIBE_MSG:
+            if len(re.findall(status, content)) > 0:
+                return True
+        raise Exception(content)
+
+    def user_subscribe(self, email, password, language='fr', first_name=u'', last_name=u''):
+
+        url = '%s/subscribe/%s' % (self.main_url, self.name)
+
+        password = check_encoding(password, self.encoding)
+        email = check_encoding(email, self.encoding)
+        first_name = check_encoding(first_name, self.encoding)
+        last_name = check_encoding(last_name, self.encoding)
+        name = '%s %s' % (first_name, last_name)
+
+        SUBSCRIBE_DATA['email'] = email
+        SUBSCRIBE_DATA['pw'] = password
+        SUBSCRIBE_DATA['pw-conf'] = password
+        SUBSCRIBE_DATA['fullname'] = name
+        SUBSCRIBE_DATA['language'] = language
+        opener = urllib2.build_opener(MultipartPostHandler(self.encoding, True))
+        request = opener.open(url, SUBSCRIBE_DATA)
+        content = request.read()
+        # no error code to process
+
+    def user_unsubscribe(self, email, language='fr'):
+
+        url = '%s/options/%s/%s' % (self.main_url, self.name, email)
+
+        email = check_encoding(email, self.encoding)
+
+        UNSUBSCRIBE_DATA['email'] = email
+        UNSUBSCRIBE_DATA['language'] = language
+        UNSUBSCRIBE_DATA['login-unsub'] = UNSUBSCRIBE_BUTTON[language]
+        
+        opener = urllib2.build_opener(MultipartPostHandler(self.encoding, True))
+        request = opener.open(url, UNSUBSCRIBE_DATA)
+        content = request.read()
+        # no error code to process
